package com.yihu.quota.controller;

import com.google.gson.Gson;
import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.constants.ServiceApi;
import com.yihu.ehr.util.rest.Envelop;
import com.yihu.quota.model.cube.Cube;
import com.yihu.quota.model.cube.CubeMapping;
import com.yihu.quota.model.cube.CubeMemberMapping;
import com.yihu.quota.model.rest.FieldInfo;
import com.yihu.quota.service.cube.CubeMappingService;
import com.yihu.quota.service.cube.CubeMemberMappingService;
import com.yihu.quota.service.cube.CubeService;
import com.yihu.quota.service.cube.ElasticSearchDataProcessService;
import com.yihu.quota.util.ElasticSearchHandler;
import com.yihu.quota.vo.CubeMappingModel;
import com.yihu.quota.vo.CubeMemberMappingModel;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang.StringUtils;
import org.elasticsearch.client.transport.TransportClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by janseny on 2018/9/19.
 */
@RestController
@RequestMapping(ApiVersion.Version1_0 + "/cube")
@Api(description = "OLAP 多维数据集 控制入口")
public class CubeController  extends BaseController {

    @Autowired
    private CubeService cubeService;
    @Autowired
    private CubeMappingService cubeMappingService;
    @Autowired
    private CubeMemberMappingService cubeMemberMappingService;
    @Autowired
    private ElasticSearchHandler esHandler;
    @Autowired
    private ElasticSearchDataProcessService elasticSearchDataProcessService;

    @ApiOperation(value = "创建多维数据集ES的mapping结构 ")
    @RequestMapping(value = "/elasticSearchMapping", method = RequestMethod.GET)
    public Envelop getQuotaReport(
            @ApiParam(name = "id", value = "多维数据集ID", required = true)
            @RequestParam(value = "id", required = true) int id
    ) {
        Envelop envelop = new Envelop();
        try {
            List<FieldInfo> fieldInfoList = new ArrayList<>();
            Cube cube = cubeService.findOne(id);
            String index = "";
            String type = "";
            if(cube.getIndexName() != null){
                index = cube.getIndexName();
            }
            if(cube.getIndexType() != null){
                type = cube.getIndexType();
            }
            boolean isExist =  esHandler.isExistsIndex(index,esHandler.getTransportClient());
            if(isExist){
                envelop.setSuccessFlg(false);
                envelop.setErrorMsg("索引已存在");
                return envelop;
            }
            // 数据集维度
            List<CubeMappingModel> cubeMappingModels = cubeMappingService.findCubeMappingModelsByCubeId(id);
            if(cubeMappingModels != null && cubeMappingModels.size() > 0){
                FieldInfo rowKeyFieldInfo =  new FieldInfo();
                rowKeyFieldInfo.setField("rowkey");
                rowKeyFieldInfo.setDataType("String");
                fieldInfoList.add(rowKeyFieldInfo);
                //extra_total 默认添加字段用于统计聚合使用
                FieldInfo totalFieldInfo =  new FieldInfo();
                totalFieldInfo.setField("extra_total");
                totalFieldInfo.setDataType("String");
                fieldInfoList.add(totalFieldInfo);


                for(CubeMappingModel cubeMappingModel : cubeMappingModels){
                    String fieldCode = cubeMappingModel.getDimensionCode();
                    //数据字典名称作为一个属性 扩展
                    if(StringUtils.isNotEmpty(cubeMappingModel.getDict())){
                        FieldInfo dictFieldName =  new FieldInfo();
                        dictFieldName.setField(fieldCode + "_name");
                        dictFieldName.setDataType("String");
                        fieldInfoList.add(dictFieldName);
                    }

                    FieldInfo fieldInfo =  new FieldInfo();
                    fieldInfo.setField(fieldCode);
                    // 子成员 扩展
                    List<CubeMemberMappingModel>  cubeMemberMappingModels = cubeMemberMappingService.findCubeMemberMappingModelsById(cubeMappingModel.getId());
                    if(cubeMemberMappingModels != null && cubeMemberMappingModels.size() > 0){
                        List<FieldInfo> childFieldiList = new ArrayList<>();
                        //如果存在子集设置 子集属性
                        if(cubeMappingModel.getChildSaveType() != null){
                            if(cubeMappingModel.getChildSaveType().equals(1)){
                                fieldInfo.setDataType("object");
                            }else if(cubeMappingModel.getChildSaveType().equals(2)){
                                fieldInfo.setDataType("nested");
                            }
                        }
                        for(CubeMemberMappingModel cubeMemberMappingModel : cubeMemberMappingModels){
                            FieldInfo childFieldInfo =  new FieldInfo();
                            childFieldInfo.setField(cubeMemberMappingModel.getDimensionCode());
                            childFieldInfo.setDataType(cubeMemberMappingModel.getDataType());
                            childFieldiList.add(childFieldInfo);
                            //数据字典名称作为一个子属性 扩展
                            if(StringUtils.isNotEmpty(cubeMemberMappingModel.getDict())){
                                FieldInfo dictFieldName =  new FieldInfo();
                                dictFieldName.setField(fieldCode + "_name");
                                dictFieldName.setDataType("String");
                                childFieldiList.add(dictFieldName);
                            }
                        }
                        fieldInfo.setFieldInfos(childFieldiList);
                    }else {
                        fieldInfo.setDataType(cubeMappingModel.getDataType());
                    }
                    fieldInfoList.add(fieldInfo);
                }
                FieldInfo updateDateFieldInfo =  new FieldInfo();
                updateDateFieldInfo.setField("update_date");
                updateDateFieldInfo.setDataType("Date");
                fieldInfoList.add(updateDateFieldInfo);
                esHandler.createIndexAndCreateMapping(index, type, fieldInfoList, esHandler.getTransportClient());
                envelop.setSuccessFlg(true);
            }else {
                envelop.setSuccessFlg(false);
                envelop.setErrorMsg("没有维度信息");
            }
            return envelop;
        } catch (Exception e) {
            e.printStackTrace();
            invalidUserException(e, -1, "创建失败:" + e.getMessage());
        }
        envelop.setSuccessFlg(false);
        return envelop;
    }

    @ApiOperation(value = "保存mapping 数据 ")
    @RequestMapping(value = "/saveElasticSearchMappingData", method = RequestMethod.GET)
    public Envelop saveElasticSearchMappingData(
            @ApiParam(name = "dataSource", value = "数据源 hbase / mysql", required = true, defaultValue = "hbase")
            @RequestParam(value = "dataSource", required = true) String dataSource,
            @ApiParam(name = "table", value = "表 Healtharchive / HealtharchiveSub", required = true)
            @RequestParam(value = "table", required = true) String table,
            @ApiParam(name = "rowkey", value = "行数据唯一键值", required = true)
            @RequestParam(value = "rowkey", required = true) String rowkey,
            @ApiParam(name = "cloumn", value = "列", required = true)
            @RequestParam(value = "cloumn", required = true) String cloumn,
            @ApiParam(name = "cloumnValue", value = "列值", required = true)
            @RequestParam(value = "cloumnValue", required = true) Object cloumnValue,
            @ApiParam(name = "profileId", value = "关联父级主表rowKey值", required = false)
            @RequestParam(value = "profileId", required = false) String profileId

    ){
        Envelop envelop = new Envelop();
        envelop.setSuccessFlg(true);
        try {
            Gson gson = new Gson();
            Map<String, Object> source  = new HashMap<>();
            source.put("dataSource", dataSource);
            source.put("table",table);
            source.put("rowkey",rowkey);
            source.put("profile_id",profileId);
            String actionStr = "Put";
            source.put("action",actionStr);
            source.put(cloumn,cloumnValue);
            String jsonData = gson.toJson(source);
            elasticSearchDataProcessService.saveData(jsonData);
        }catch (Exception e){
            e.printStackTrace();
            envelop.setSuccessFlg(false);
        }
        return envelop;
    }


    @ApiOperation(value = "删除mapping 数据 ")
    @RequestMapping(value = "/delElasticSearchMappingData", method = RequestMethod.GET)
    public Envelop delElasticSearchMappingData(
            @ApiParam(name = "dataSource", value = "数据源 hbase / mysql", required = true, defaultValue = "hbase")
            @RequestParam(value = "dataSource", required = true) String dataSource,
            @ApiParam(name = "table", value = "表 Healtharchive / HealtharchiveSub", required = true)
            @RequestParam(value = "table", required = true) String table,
            @ApiParam(name = "rowkey", value = "行数据唯一键值", required = true)
            @RequestParam(value = "rowkey", required = true) String rowkey
    ){
        Envelop envelop = new Envelop();
        envelop.setSuccessFlg(true);
        try {
            Gson gson = new Gson();
            Map<String, Object> source  = new HashMap<>();
            source.put("dataSource", dataSource);
            source.put("table",table);
            source.put("rowkey",rowkey);
            source.put("action","DeleteFamily");
            String jsonData = gson.toJson(source);
            elasticSearchDataProcessService.saveData(jsonData);
        }catch (Exception e){
            e.printStackTrace();
            envelop.setSuccessFlg(false);
        }
        return envelop;
    }

}
