package com.yihu.quota.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.constants.ApiVersion;
import com.yihu.ehr.util.rest.Envelop;
import com.yihu.quota.model.source.DataSourcesTableField;
import com.yihu.quota.service.source.DataSourcesTableFieldService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.net.URLDecoder;
import java.util.List;

/**
 * Created by wxw on 2018/9/10.
 *
 * @author wxw.
 */
@RestController
@RequestMapping(ApiVersion.Version1_0)
@Api(value = "DataSourcesTableFieldController", description = "OLAP-数据源表字段对应表", tags = {"OLAP-数据源表字段对应表"})
public class DataSourcesTableFieldController {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private DataSourcesTableFieldService dataSourcesFieldService;

    @RequestMapping(value = "/olap/searchDSField", method = RequestMethod.GET)
    @ApiOperation(value = "根据查询条件查询,具有分页功能;filters='id=1'")
    public Envelop searchDSField(
            @ApiParam(name = "fields", value = "返回的字段，为空返回全部字段", defaultValue = "")
            @RequestParam(value = "fields", required = false) String fields,
            @ApiParam(name = "filters", value = "过滤器，为空检索所有条件")
            @RequestParam(value = "filters", required = false) String filters,
            @ApiParam(name = "sorts", value = "排序，规则参见说明文档", defaultValue = "+tableId")
            @RequestParam(value = "sorts", required = false) String sorts,
            @ApiParam(name = "size", value = "分页大小", defaultValue = "15")
            @RequestParam(value = "size", required = false) int size,
            @ApiParam(name = "page", value = "页码", defaultValue = "1")
            @RequestParam(value = "page", required = false) int page) throws Exception {
        Envelop envelop = new Envelop();
        List<DataSourcesTableField> search = dataSourcesFieldService.search(fields, filters, sorts, page, size);
        envelop.setSuccessFlg(true);
        envelop.setDetailModelList(search);
        return envelop;
    }

    @RequestMapping(value = "/olap/saveOrUpdateDSField", method = RequestMethod.POST)
    @ApiOperation(value = "添加或修改操作")
    public Envelop saveOrUpdateDSField(
            @ApiParam(name = "model", value = "json数据模型", defaultValue = "")
            @RequestParam String model) throws Exception {
        Envelop envelop = new Envelop();
        DataSourcesTableField dimensionMember = objectMapper.readValue(URLDecoder.decode(model, "UTF-8"), DataSourcesTableField.class);
        // 判断字段类型是否是允许的类型
        boolean allowType = dataSourcesFieldService.isAllowType(dimensionMember.getFieldType());
        if (!allowType) {
            // 不是允许的类型
            envelop.setSuccessFlg(false);
            envelop.setErrorMsg("不被允许的字段类型");
            return envelop;
        }
        if (null == dimensionMember.getId()) {
            // 新增
            dataSourcesFieldService.save(dimensionMember);
        } else {
            dataSourcesFieldService.save(dimensionMember);
        }
        envelop.setSuccessFlg(true);
        return envelop;
    }

    @RequestMapping(value = "/olap/deleteDSFieldById", method = RequestMethod.DELETE)
    @ApiOperation(value = "根据id删除")
    public Envelop deleteOlapDMById(
            @ApiParam(name = "id", value = "编号", defaultValue = "")
            @RequestParam(value = "id") Integer id) throws Exception {
        Envelop envelop = new Envelop();
        dataSourcesFieldService.delete(id);
        envelop.setSuccessFlg(true);
        return envelop;
    }
}
