package com.yihu.quota.dao.view;

import com.yihu.quota.model.view.ViewDimension;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.repository.query.Param;

import java.util.List;


/**
 * 视图维度 DAO
 *
 * @author janseny
 * @date 2018年9月26日
 */
public interface ViewDimensionDao extends PagingAndSortingRepository<ViewDimension, String> {

    /**
     * 获取各组组内的顶层行维度
     *
     * @param viewId
     * @return
     */
    @Query(" FROM ViewDimension WHERE viewId = :viewId AND dimensionType = 'row' AND groupRowOrder = 1 ORDER BY groupRow ASC ")
    List<ViewDimension> getGroupTopRowDimensionList(@Param("viewId") String viewId);

    /**
     * 获取组内的其他行维度
     *
     * @param viewId
     * @param groupRow
     * @return
     */
    @Query(" FROM ViewDimension WHERE viewId = :viewId AND groupRow = :groupRow AND dimensionType = 'row' AND groupRowOrder <> 1 ORDER BY groupRowOrder ASC ")
    List<ViewDimension> getGroupOtherRowDimensionList(@Param("viewId") String viewId, @Param("groupRow") Integer groupRow);

    /**
     * 获取列维度
     *
     * @param viewId
     * @return
     */
    @Query(" FROM ViewDimension WHERE viewId = :viewId AND dimensionType = 'col' ORDER BY columnDrillOrder ASC ")
    List<ViewDimension> getColDimensionList(@Param("viewId") String viewId);

}
